# frozen_string_literal: true

# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Auto-generated by gapic-generator-ruby. DO NOT EDIT!

require "google/cloud/errors"
require "google/cloud/orgpolicy/v2/orgpolicy_pb"

module Google
  module Cloud
    module OrgPolicy
      module V2
        module OrgPolicy
          ##
          # Client for the OrgPolicy service.
          #
          # An interface for managing organization policies.
          #
          # The Cloud Org Policy service provides a simple mechanism for organizations to
          # restrict the allowed configurations across their entire Cloud Resource
          # hierarchy.
          #
          # You can use a `policy` to configure restrictions in Cloud resources. For
          # example, you can enforce a `policy` that restricts which Google
          # Cloud Platform APIs can be activated in a certain part of your resource
          # hierarchy, or prevents serial port access to VM instances in a particular
          # folder.
          #
          # `Policies` are inherited down through the resource hierarchy. A `policy`
          # applied to a parent resource automatically applies to all its child resources
          # unless overridden with a `policy` lower in the hierarchy.
          #
          # A `constraint` defines an aspect of a resource's configuration that can be
          # controlled by an organization's policy administrator. `Policies` are a
          # collection of `constraints` that defines their allowable configuration on a
          # particular resource and its child resources.
          #
          class Client
            include Paths

            # @private
            attr_reader :org_policy_stub

            ##
            # Configure the OrgPolicy Client class.
            #
            # See {::Google::Cloud::OrgPolicy::V2::OrgPolicy::Client::Configuration}
            # for a description of the configuration fields.
            #
            # ## Example
            #
            # To modify the configuration for all OrgPolicy clients:
            #
            #     ::Google::Cloud::OrgPolicy::V2::OrgPolicy::Client.configure do |config|
            #       config.timeout = 10.0
            #     end
            #
            # @yield [config] Configure the Client client.
            # @yieldparam config [Client::Configuration]
            #
            # @return [Client::Configuration]
            #
            def self.configure
              @configure ||= begin
                namespace = ["Google", "Cloud", "OrgPolicy", "V2"]
                parent_config = while namespace.any?
                                  parent_name = namespace.join "::"
                                  parent_const = const_get parent_name
                                  break parent_const.configure if parent_const.respond_to? :configure
                                  namespace.pop
                                end
                default_config = Client::Configuration.new parent_config

                default_config.rpcs.list_constraints.timeout = 60.0
                default_config.rpcs.list_constraints.retry_policy = {
                  initial_delay: 1.0,
                  max_delay: 10.0,
                  multiplier: 1.3,
                  retry_codes: [14, 4]
                }

                default_config.rpcs.list_policies.timeout = 60.0
                default_config.rpcs.list_policies.retry_policy = {
                  initial_delay: 1.0,
                  max_delay: 10.0,
                  multiplier: 1.3,
                  retry_codes: [14, 4]
                }

                default_config.rpcs.get_policy.timeout = 60.0
                default_config.rpcs.get_policy.retry_policy = {
                  initial_delay: 1.0,
                  max_delay: 10.0,
                  multiplier: 1.3,
                  retry_codes: [14, 4]
                }

                default_config.rpcs.get_effective_policy.timeout = 60.0
                default_config.rpcs.get_effective_policy.retry_policy = {
                  initial_delay: 1.0,
                  max_delay: 10.0,
                  multiplier: 1.3,
                  retry_codes: [14, 4]
                }

                default_config.rpcs.create_policy.timeout = 60.0
                default_config.rpcs.create_policy.retry_policy = {
                  initial_delay: 1.0,
                  max_delay: 10.0,
                  multiplier: 1.3,
                  retry_codes: [14, 4]
                }

                default_config.rpcs.update_policy.timeout = 60.0
                default_config.rpcs.update_policy.retry_policy = {
                  initial_delay: 1.0,
                  max_delay: 10.0,
                  multiplier: 1.3,
                  retry_codes: [14, 4]
                }

                default_config.rpcs.delete_policy.timeout = 60.0
                default_config.rpcs.delete_policy.retry_policy = {
                  initial_delay: 1.0,
                  max_delay: 10.0,
                  multiplier: 1.3,
                  retry_codes: [14, 4]
                }

                default_config
              end
              yield @configure if block_given?
              @configure
            end

            ##
            # Configure the OrgPolicy Client instance.
            #
            # The configuration is set to the derived mode, meaning that values can be changed,
            # but structural changes (adding new fields, etc.) are not allowed. Structural changes
            # should be made on {Client.configure}.
            #
            # See {::Google::Cloud::OrgPolicy::V2::OrgPolicy::Client::Configuration}
            # for a description of the configuration fields.
            #
            # @yield [config] Configure the Client client.
            # @yieldparam config [Client::Configuration]
            #
            # @return [Client::Configuration]
            #
            def configure
              yield @config if block_given?
              @config
            end

            ##
            # Create a new OrgPolicy client object.
            #
            # ## Examples
            #
            # To create a new OrgPolicy client with the default
            # configuration:
            #
            #     client = ::Google::Cloud::OrgPolicy::V2::OrgPolicy::Client.new
            #
            # To create a new OrgPolicy client with a custom
            # configuration:
            #
            #     client = ::Google::Cloud::OrgPolicy::V2::OrgPolicy::Client.new do |config|
            #       config.timeout = 10.0
            #     end
            #
            # @yield [config] Configure the OrgPolicy client.
            # @yieldparam config [Client::Configuration]
            #
            def initialize
              # These require statements are intentionally placed here to initialize
              # the gRPC module only when it's required.
              # See https://github.com/googleapis/toolkit/issues/446
              require "gapic/grpc"
              require "google/cloud/orgpolicy/v2/orgpolicy_services_pb"

              # Create the configuration object
              @config = Configuration.new Client.configure

              # Yield the configuration if needed
              yield @config if block_given?

              # Create credentials
              credentials = @config.credentials
              # Use self-signed JWT if the scope and endpoint are unchanged from default,
              # but only if the default endpoint does not have a region prefix.
              enable_self_signed_jwt = @config.scope == Client.configure.scope &&
                                       @config.endpoint == Client.configure.endpoint &&
                                       !@config.endpoint.split(".").first.include?("-")
              credentials ||= Credentials.default scope: @config.scope,
                                                  enable_self_signed_jwt: enable_self_signed_jwt
              if credentials.is_a?(String) || credentials.is_a?(Hash)
                credentials = Credentials.new credentials, scope: @config.scope
              end
              @quota_project_id = @config.quota_project
              @quota_project_id ||= credentials.quota_project_id if credentials.respond_to? :quota_project_id

              @org_policy_stub = ::Gapic::ServiceStub.new(
                ::Google::Cloud::OrgPolicy::V2::OrgPolicy::Stub,
                credentials:  credentials,
                endpoint:     @config.endpoint,
                channel_args: @config.channel_args,
                interceptors: @config.interceptors
              )
            end

            # Service calls

            ##
            # Lists `Constraints` that could be applied on the specified resource.
            #
            # @overload list_constraints(request, options = nil)
            #   Pass arguments to `list_constraints` via a request object, either of type
            #   {::Google::Cloud::OrgPolicy::V2::ListConstraintsRequest} or an equivalent Hash.
            #
            #   @param request [::Google::Cloud::OrgPolicy::V2::ListConstraintsRequest, ::Hash]
            #     A request object representing the call parameters. Required. To specify no
            #     parameters, or to keep all the default parameter values, pass an empty Hash.
            #   @param options [::Gapic::CallOptions, ::Hash]
            #     Overrides the default settings for this call, e.g, timeout, retries, etc. Optional.
            #
            # @overload list_constraints(parent: nil, page_size: nil, page_token: nil)
            #   Pass arguments to `list_constraints` via keyword arguments. Note that at
            #   least one keyword argument is required. To specify no parameters, or to keep all
            #   the default parameter values, pass an empty Hash as a request object (see above).
            #
            #   @param parent [::String]
            #     Required. The Cloud resource that parents the constraint. Must be in one of the
            #     following forms:
            #     * `projects/{project_number}`
            #     * `projects/{project_id}`
            #     * `folders/{folder_id}`
            #     * `organizations/{organization_id}`
            #   @param page_size [::Integer]
            #     Size of the pages to be returned. This is currently unsupported and will
            #     be ignored. The server may at any point start using this field to limit
            #     page size.
            #   @param page_token [::String]
            #     Page token used to retrieve the next page. This is currently unsupported
            #     and will be ignored. The server may at any point start using this field.
            #
            # @yield [response, operation] Access the result along with the RPC operation
            # @yieldparam response [::Gapic::PagedEnumerable<::Google::Cloud::OrgPolicy::V2::Constraint>]
            # @yieldparam operation [::GRPC::ActiveCall::Operation]
            #
            # @return [::Gapic::PagedEnumerable<::Google::Cloud::OrgPolicy::V2::Constraint>]
            #
            # @raise [::Google::Cloud::Error] if the RPC is aborted.
            #
            def list_constraints request, options = nil
              raise ::ArgumentError, "request must be provided" if request.nil?

              request = ::Gapic::Protobuf.coerce request, to: ::Google::Cloud::OrgPolicy::V2::ListConstraintsRequest

              # Converts hash and nil to an options object
              options = ::Gapic::CallOptions.new(**options.to_h) if options.respond_to? :to_h

              # Customize the options with defaults
              metadata = @config.rpcs.list_constraints.metadata.to_h

              # Set x-goog-api-client and x-goog-user-project headers
              metadata[:"x-goog-api-client"] ||= ::Gapic::Headers.x_goog_api_client \
                lib_name: @config.lib_name, lib_version: @config.lib_version,
                gapic_version: ::Google::Cloud::OrgPolicy::V2::VERSION
              metadata[:"x-goog-user-project"] = @quota_project_id if @quota_project_id

              header_params = {
                "parent" => request.parent
              }
              request_params_header = header_params.map { |k, v| "#{k}=#{v}" }.join("&")
              metadata[:"x-goog-request-params"] ||= request_params_header

              options.apply_defaults timeout:      @config.rpcs.list_constraints.timeout,
                                     metadata:     metadata,
                                     retry_policy: @config.rpcs.list_constraints.retry_policy
              options.apply_defaults metadata:     @config.metadata,
                                     retry_policy: @config.retry_policy

              @org_policy_stub.call_rpc :list_constraints, request, options: options do |response, operation|
                response = ::Gapic::PagedEnumerable.new @org_policy_stub, :list_constraints, request, response, operation, options
                yield response, operation if block_given?
                return response
              end
            rescue ::GRPC::BadStatus => e
              raise ::Google::Cloud::Error.from_error(e)
            end

            ##
            # Retrieves all of the `Policies` that exist on a particular resource.
            #
            # @overload list_policies(request, options = nil)
            #   Pass arguments to `list_policies` via a request object, either of type
            #   {::Google::Cloud::OrgPolicy::V2::ListPoliciesRequest} or an equivalent Hash.
            #
            #   @param request [::Google::Cloud::OrgPolicy::V2::ListPoliciesRequest, ::Hash]
            #     A request object representing the call parameters. Required. To specify no
            #     parameters, or to keep all the default parameter values, pass an empty Hash.
            #   @param options [::Gapic::CallOptions, ::Hash]
            #     Overrides the default settings for this call, e.g, timeout, retries, etc. Optional.
            #
            # @overload list_policies(parent: nil, page_size: nil, page_token: nil)
            #   Pass arguments to `list_policies` via keyword arguments. Note that at
            #   least one keyword argument is required. To specify no parameters, or to keep all
            #   the default parameter values, pass an empty Hash as a request object (see above).
            #
            #   @param parent [::String]
            #     Required. The target Cloud resource that parents the set of constraints and policies
            #     that will be returned from this call. Must be in one of the following
            #     forms:
            #     * `projects/{project_number}`
            #     * `projects/{project_id}`
            #     * `folders/{folder_id}`
            #     * `organizations/{organization_id}`
            #   @param page_size [::Integer]
            #     Size of the pages to be returned. This is currently unsupported and will
            #     be ignored. The server may at any point start using this field to limit
            #     page size.
            #   @param page_token [::String]
            #     Page token used to retrieve the next page. This is currently unsupported
            #     and will be ignored. The server may at any point start using this field.
            #
            # @yield [response, operation] Access the result along with the RPC operation
            # @yieldparam response [::Gapic::PagedEnumerable<::Google::Cloud::OrgPolicy::V2::Policy>]
            # @yieldparam operation [::GRPC::ActiveCall::Operation]
            #
            # @return [::Gapic::PagedEnumerable<::Google::Cloud::OrgPolicy::V2::Policy>]
            #
            # @raise [::Google::Cloud::Error] if the RPC is aborted.
            #
            def list_policies request, options = nil
              raise ::ArgumentError, "request must be provided" if request.nil?

              request = ::Gapic::Protobuf.coerce request, to: ::Google::Cloud::OrgPolicy::V2::ListPoliciesRequest

              # Converts hash and nil to an options object
              options = ::Gapic::CallOptions.new(**options.to_h) if options.respond_to? :to_h

              # Customize the options with defaults
              metadata = @config.rpcs.list_policies.metadata.to_h

              # Set x-goog-api-client and x-goog-user-project headers
              metadata[:"x-goog-api-client"] ||= ::Gapic::Headers.x_goog_api_client \
                lib_name: @config.lib_name, lib_version: @config.lib_version,
                gapic_version: ::Google::Cloud::OrgPolicy::V2::VERSION
              metadata[:"x-goog-user-project"] = @quota_project_id if @quota_project_id

              header_params = {
                "parent" => request.parent
              }
              request_params_header = header_params.map { |k, v| "#{k}=#{v}" }.join("&")
              metadata[:"x-goog-request-params"] ||= request_params_header

              options.apply_defaults timeout:      @config.rpcs.list_policies.timeout,
                                     metadata:     metadata,
                                     retry_policy: @config.rpcs.list_policies.retry_policy
              options.apply_defaults metadata:     @config.metadata,
                                     retry_policy: @config.retry_policy

              @org_policy_stub.call_rpc :list_policies, request, options: options do |response, operation|
                response = ::Gapic::PagedEnumerable.new @org_policy_stub, :list_policies, request, response, operation, options
                yield response, operation if block_given?
                return response
              end
            rescue ::GRPC::BadStatus => e
              raise ::Google::Cloud::Error.from_error(e)
            end

            ##
            # Gets a `Policy` on a resource.
            #
            # If no `Policy` is set on the resource, NOT_FOUND is returned. The
            # `etag` value can be used with `UpdatePolicy()` to update a
            # `Policy` during read-modify-write.
            #
            # @overload get_policy(request, options = nil)
            #   Pass arguments to `get_policy` via a request object, either of type
            #   {::Google::Cloud::OrgPolicy::V2::GetPolicyRequest} or an equivalent Hash.
            #
            #   @param request [::Google::Cloud::OrgPolicy::V2::GetPolicyRequest, ::Hash]
            #     A request object representing the call parameters. Required. To specify no
            #     parameters, or to keep all the default parameter values, pass an empty Hash.
            #   @param options [::Gapic::CallOptions, ::Hash]
            #     Overrides the default settings for this call, e.g, timeout, retries, etc. Optional.
            #
            # @overload get_policy(name: nil)
            #   Pass arguments to `get_policy` via keyword arguments. Note that at
            #   least one keyword argument is required. To specify no parameters, or to keep all
            #   the default parameter values, pass an empty Hash as a request object (see above).
            #
            #   @param name [::String]
            #     Required. Resource name of the policy. See `Policy` for naming requirements.
            #
            # @yield [response, operation] Access the result along with the RPC operation
            # @yieldparam response [::Google::Cloud::OrgPolicy::V2::Policy]
            # @yieldparam operation [::GRPC::ActiveCall::Operation]
            #
            # @return [::Google::Cloud::OrgPolicy::V2::Policy]
            #
            # @raise [::Google::Cloud::Error] if the RPC is aborted.
            #
            def get_policy request, options = nil
              raise ::ArgumentError, "request must be provided" if request.nil?

              request = ::Gapic::Protobuf.coerce request, to: ::Google::Cloud::OrgPolicy::V2::GetPolicyRequest

              # Converts hash and nil to an options object
              options = ::Gapic::CallOptions.new(**options.to_h) if options.respond_to? :to_h

              # Customize the options with defaults
              metadata = @config.rpcs.get_policy.metadata.to_h

              # Set x-goog-api-client and x-goog-user-project headers
              metadata[:"x-goog-api-client"] ||= ::Gapic::Headers.x_goog_api_client \
                lib_name: @config.lib_name, lib_version: @config.lib_version,
                gapic_version: ::Google::Cloud::OrgPolicy::V2::VERSION
              metadata[:"x-goog-user-project"] = @quota_project_id if @quota_project_id

              header_params = {
                "name" => request.name
              }
              request_params_header = header_params.map { |k, v| "#{k}=#{v}" }.join("&")
              metadata[:"x-goog-request-params"] ||= request_params_header

              options.apply_defaults timeout:      @config.rpcs.get_policy.timeout,
                                     metadata:     metadata,
                                     retry_policy: @config.rpcs.get_policy.retry_policy
              options.apply_defaults metadata:     @config.metadata,
                                     retry_policy: @config.retry_policy

              @org_policy_stub.call_rpc :get_policy, request, options: options do |response, operation|
                yield response, operation if block_given?
                return response
              end
            rescue ::GRPC::BadStatus => e
              raise ::Google::Cloud::Error.from_error(e)
            end

            ##
            # Gets the effective `Policy` on a resource. This is the result of merging
            # `Policies` in the resource hierarchy and evaluating conditions. The
            # returned `Policy` will not have an `etag` or `condition` set because it is
            # a computed `Policy` across multiple resources.
            # Subtrees of Resource Manager resource hierarchy with 'under:' prefix will
            # not be expanded.
            #
            # @overload get_effective_policy(request, options = nil)
            #   Pass arguments to `get_effective_policy` via a request object, either of type
            #   {::Google::Cloud::OrgPolicy::V2::GetEffectivePolicyRequest} or an equivalent Hash.
            #
            #   @param request [::Google::Cloud::OrgPolicy::V2::GetEffectivePolicyRequest, ::Hash]
            #     A request object representing the call parameters. Required. To specify no
            #     parameters, or to keep all the default parameter values, pass an empty Hash.
            #   @param options [::Gapic::CallOptions, ::Hash]
            #     Overrides the default settings for this call, e.g, timeout, retries, etc. Optional.
            #
            # @overload get_effective_policy(name: nil)
            #   Pass arguments to `get_effective_policy` via keyword arguments. Note that at
            #   least one keyword argument is required. To specify no parameters, or to keep all
            #   the default parameter values, pass an empty Hash as a request object (see above).
            #
            #   @param name [::String]
            #     Required. The effective policy to compute. See `Policy` for naming rules.
            #
            # @yield [response, operation] Access the result along with the RPC operation
            # @yieldparam response [::Google::Cloud::OrgPolicy::V2::Policy]
            # @yieldparam operation [::GRPC::ActiveCall::Operation]
            #
            # @return [::Google::Cloud::OrgPolicy::V2::Policy]
            #
            # @raise [::Google::Cloud::Error] if the RPC is aborted.
            #
            def get_effective_policy request, options = nil
              raise ::ArgumentError, "request must be provided" if request.nil?

              request = ::Gapic::Protobuf.coerce request, to: ::Google::Cloud::OrgPolicy::V2::GetEffectivePolicyRequest

              # Converts hash and nil to an options object
              options = ::Gapic::CallOptions.new(**options.to_h) if options.respond_to? :to_h

              # Customize the options with defaults
              metadata = @config.rpcs.get_effective_policy.metadata.to_h

              # Set x-goog-api-client and x-goog-user-project headers
              metadata[:"x-goog-api-client"] ||= ::Gapic::Headers.x_goog_api_client \
                lib_name: @config.lib_name, lib_version: @config.lib_version,
                gapic_version: ::Google::Cloud::OrgPolicy::V2::VERSION
              metadata[:"x-goog-user-project"] = @quota_project_id if @quota_project_id

              header_params = {
                "name" => request.name
              }
              request_params_header = header_params.map { |k, v| "#{k}=#{v}" }.join("&")
              metadata[:"x-goog-request-params"] ||= request_params_header

              options.apply_defaults timeout:      @config.rpcs.get_effective_policy.timeout,
                                     metadata:     metadata,
                                     retry_policy: @config.rpcs.get_effective_policy.retry_policy
              options.apply_defaults metadata:     @config.metadata,
                                     retry_policy: @config.retry_policy

              @org_policy_stub.call_rpc :get_effective_policy, request, options: options do |response, operation|
                yield response, operation if block_given?
                return response
              end
            rescue ::GRPC::BadStatus => e
              raise ::Google::Cloud::Error.from_error(e)
            end

            ##
            # Creates a Policy.
            #
            # Returns a `google.rpc.Status` with `google.rpc.Code.NOT_FOUND` if the
            # constraint does not exist.
            # Returns a `google.rpc.Status` with `google.rpc.Code.ALREADY_EXISTS` if the
            # policy already exists on the given Cloud resource.
            #
            # @overload create_policy(request, options = nil)
            #   Pass arguments to `create_policy` via a request object, either of type
            #   {::Google::Cloud::OrgPolicy::V2::CreatePolicyRequest} or an equivalent Hash.
            #
            #   @param request [::Google::Cloud::OrgPolicy::V2::CreatePolicyRequest, ::Hash]
            #     A request object representing the call parameters. Required. To specify no
            #     parameters, or to keep all the default parameter values, pass an empty Hash.
            #   @param options [::Gapic::CallOptions, ::Hash]
            #     Overrides the default settings for this call, e.g, timeout, retries, etc. Optional.
            #
            # @overload create_policy(parent: nil, policy: nil)
            #   Pass arguments to `create_policy` via keyword arguments. Note that at
            #   least one keyword argument is required. To specify no parameters, or to keep all
            #   the default parameter values, pass an empty Hash as a request object (see above).
            #
            #   @param parent [::String]
            #     Required. The Cloud resource that will parent the new Policy. Must be in one of the
            #     following forms:
            #     * `projects/{project_number}`
            #     * `projects/{project_id}`
            #     * `folders/{folder_id}`
            #     * `organizations/{organization_id}`
            #   @param policy [::Google::Cloud::OrgPolicy::V2::Policy, ::Hash]
            #     Required. `Policy` to create.
            #
            # @yield [response, operation] Access the result along with the RPC operation
            # @yieldparam response [::Google::Cloud::OrgPolicy::V2::Policy]
            # @yieldparam operation [::GRPC::ActiveCall::Operation]
            #
            # @return [::Google::Cloud::OrgPolicy::V2::Policy]
            #
            # @raise [::Google::Cloud::Error] if the RPC is aborted.
            #
            def create_policy request, options = nil
              raise ::ArgumentError, "request must be provided" if request.nil?

              request = ::Gapic::Protobuf.coerce request, to: ::Google::Cloud::OrgPolicy::V2::CreatePolicyRequest

              # Converts hash and nil to an options object
              options = ::Gapic::CallOptions.new(**options.to_h) if options.respond_to? :to_h

              # Customize the options with defaults
              metadata = @config.rpcs.create_policy.metadata.to_h

              # Set x-goog-api-client and x-goog-user-project headers
              metadata[:"x-goog-api-client"] ||= ::Gapic::Headers.x_goog_api_client \
                lib_name: @config.lib_name, lib_version: @config.lib_version,
                gapic_version: ::Google::Cloud::OrgPolicy::V2::VERSION
              metadata[:"x-goog-user-project"] = @quota_project_id if @quota_project_id

              header_params = {
                "parent" => request.parent
              }
              request_params_header = header_params.map { |k, v| "#{k}=#{v}" }.join("&")
              metadata[:"x-goog-request-params"] ||= request_params_header

              options.apply_defaults timeout:      @config.rpcs.create_policy.timeout,
                                     metadata:     metadata,
                                     retry_policy: @config.rpcs.create_policy.retry_policy
              options.apply_defaults metadata:     @config.metadata,
                                     retry_policy: @config.retry_policy

              @org_policy_stub.call_rpc :create_policy, request, options: options do |response, operation|
                yield response, operation if block_given?
                return response
              end
            rescue ::GRPC::BadStatus => e
              raise ::Google::Cloud::Error.from_error(e)
            end

            ##
            # Updates a Policy.
            #
            # Returns a `google.rpc.Status` with `google.rpc.Code.NOT_FOUND` if the
            # constraint or the policy do not exist.
            # Returns a `google.rpc.Status` with `google.rpc.Code.ABORTED` if the etag
            # supplied in the request does not match the persisted etag of the policy
            #
            # Note: the supplied policy will perform a full overwrite of all
            # fields.
            #
            # @overload update_policy(request, options = nil)
            #   Pass arguments to `update_policy` via a request object, either of type
            #   {::Google::Cloud::OrgPolicy::V2::UpdatePolicyRequest} or an equivalent Hash.
            #
            #   @param request [::Google::Cloud::OrgPolicy::V2::UpdatePolicyRequest, ::Hash]
            #     A request object representing the call parameters. Required. To specify no
            #     parameters, or to keep all the default parameter values, pass an empty Hash.
            #   @param options [::Gapic::CallOptions, ::Hash]
            #     Overrides the default settings for this call, e.g, timeout, retries, etc. Optional.
            #
            # @overload update_policy(policy: nil)
            #   Pass arguments to `update_policy` via keyword arguments. Note that at
            #   least one keyword argument is required. To specify no parameters, or to keep all
            #   the default parameter values, pass an empty Hash as a request object (see above).
            #
            #   @param policy [::Google::Cloud::OrgPolicy::V2::Policy, ::Hash]
            #     Required. `Policy` to update.
            #
            # @yield [response, operation] Access the result along with the RPC operation
            # @yieldparam response [::Google::Cloud::OrgPolicy::V2::Policy]
            # @yieldparam operation [::GRPC::ActiveCall::Operation]
            #
            # @return [::Google::Cloud::OrgPolicy::V2::Policy]
            #
            # @raise [::Google::Cloud::Error] if the RPC is aborted.
            #
            def update_policy request, options = nil
              raise ::ArgumentError, "request must be provided" if request.nil?

              request = ::Gapic::Protobuf.coerce request, to: ::Google::Cloud::OrgPolicy::V2::UpdatePolicyRequest

              # Converts hash and nil to an options object
              options = ::Gapic::CallOptions.new(**options.to_h) if options.respond_to? :to_h

              # Customize the options with defaults
              metadata = @config.rpcs.update_policy.metadata.to_h

              # Set x-goog-api-client and x-goog-user-project headers
              metadata[:"x-goog-api-client"] ||= ::Gapic::Headers.x_goog_api_client \
                lib_name: @config.lib_name, lib_version: @config.lib_version,
                gapic_version: ::Google::Cloud::OrgPolicy::V2::VERSION
              metadata[:"x-goog-user-project"] = @quota_project_id if @quota_project_id

              header_params = {
                "policy.name" => request.policy.name
              }
              request_params_header = header_params.map { |k, v| "#{k}=#{v}" }.join("&")
              metadata[:"x-goog-request-params"] ||= request_params_header

              options.apply_defaults timeout:      @config.rpcs.update_policy.timeout,
                                     metadata:     metadata,
                                     retry_policy: @config.rpcs.update_policy.retry_policy
              options.apply_defaults metadata:     @config.metadata,
                                     retry_policy: @config.retry_policy

              @org_policy_stub.call_rpc :update_policy, request, options: options do |response, operation|
                yield response, operation if block_given?
                return response
              end
            rescue ::GRPC::BadStatus => e
              raise ::Google::Cloud::Error.from_error(e)
            end

            ##
            # Deletes a Policy.
            #
            # Returns a `google.rpc.Status` with `google.rpc.Code.NOT_FOUND` if the
            # constraint or Org Policy does not exist.
            #
            # @overload delete_policy(request, options = nil)
            #   Pass arguments to `delete_policy` via a request object, either of type
            #   {::Google::Cloud::OrgPolicy::V2::DeletePolicyRequest} or an equivalent Hash.
            #
            #   @param request [::Google::Cloud::OrgPolicy::V2::DeletePolicyRequest, ::Hash]
            #     A request object representing the call parameters. Required. To specify no
            #     parameters, or to keep all the default parameter values, pass an empty Hash.
            #   @param options [::Gapic::CallOptions, ::Hash]
            #     Overrides the default settings for this call, e.g, timeout, retries, etc. Optional.
            #
            # @overload delete_policy(name: nil)
            #   Pass arguments to `delete_policy` via keyword arguments. Note that at
            #   least one keyword argument is required. To specify no parameters, or to keep all
            #   the default parameter values, pass an empty Hash as a request object (see above).
            #
            #   @param name [::String]
            #     Required. Name of the policy to delete.
            #     See `Policy` for naming rules.
            #
            # @yield [response, operation] Access the result along with the RPC operation
            # @yieldparam response [::Google::Protobuf::Empty]
            # @yieldparam operation [::GRPC::ActiveCall::Operation]
            #
            # @return [::Google::Protobuf::Empty]
            #
            # @raise [::Google::Cloud::Error] if the RPC is aborted.
            #
            def delete_policy request, options = nil
              raise ::ArgumentError, "request must be provided" if request.nil?

              request = ::Gapic::Protobuf.coerce request, to: ::Google::Cloud::OrgPolicy::V2::DeletePolicyRequest

              # Converts hash and nil to an options object
              options = ::Gapic::CallOptions.new(**options.to_h) if options.respond_to? :to_h

              # Customize the options with defaults
              metadata = @config.rpcs.delete_policy.metadata.to_h

              # Set x-goog-api-client and x-goog-user-project headers
              metadata[:"x-goog-api-client"] ||= ::Gapic::Headers.x_goog_api_client \
                lib_name: @config.lib_name, lib_version: @config.lib_version,
                gapic_version: ::Google::Cloud::OrgPolicy::V2::VERSION
              metadata[:"x-goog-user-project"] = @quota_project_id if @quota_project_id

              header_params = {
                "name" => request.name
              }
              request_params_header = header_params.map { |k, v| "#{k}=#{v}" }.join("&")
              metadata[:"x-goog-request-params"] ||= request_params_header

              options.apply_defaults timeout:      @config.rpcs.delete_policy.timeout,
                                     metadata:     metadata,
                                     retry_policy: @config.rpcs.delete_policy.retry_policy
              options.apply_defaults metadata:     @config.metadata,
                                     retry_policy: @config.retry_policy

              @org_policy_stub.call_rpc :delete_policy, request, options: options do |response, operation|
                yield response, operation if block_given?
                return response
              end
            rescue ::GRPC::BadStatus => e
              raise ::Google::Cloud::Error.from_error(e)
            end

            ##
            # Configuration class for the OrgPolicy API.
            #
            # This class represents the configuration for OrgPolicy,
            # providing control over timeouts, retry behavior, logging, transport
            # parameters, and other low-level controls. Certain parameters can also be
            # applied individually to specific RPCs. See
            # {::Google::Cloud::OrgPolicy::V2::OrgPolicy::Client::Configuration::Rpcs}
            # for a list of RPCs that can be configured independently.
            #
            # Configuration can be applied globally to all clients, or to a single client
            # on construction.
            #
            # # Examples
            #
            # To modify the global config, setting the timeout for list_constraints
            # to 20 seconds, and all remaining timeouts to 10 seconds:
            #
            #     ::Google::Cloud::OrgPolicy::V2::OrgPolicy::Client.configure do |config|
            #       config.timeout = 10.0
            #       config.rpcs.list_constraints.timeout = 20.0
            #     end
            #
            # To apply the above configuration only to a new client:
            #
            #     client = ::Google::Cloud::OrgPolicy::V2::OrgPolicy::Client.new do |config|
            #       config.timeout = 10.0
            #       config.rpcs.list_constraints.timeout = 20.0
            #     end
            #
            # @!attribute [rw] endpoint
            #   The hostname or hostname:port of the service endpoint.
            #   Defaults to `"orgpolicy.googleapis.com"`.
            #   @return [::String]
            # @!attribute [rw] credentials
            #   Credentials to send with calls. You may provide any of the following types:
            #    *  (`String`) The path to a service account key file in JSON format
            #    *  (`Hash`) A service account key as a Hash
            #    *  (`Google::Auth::Credentials`) A googleauth credentials object
            #       (see the [googleauth docs](https://googleapis.dev/ruby/googleauth/latest/index.html))
            #    *  (`Signet::OAuth2::Client`) A signet oauth2 client object
            #       (see the [signet docs](https://googleapis.dev/ruby/signet/latest/Signet/OAuth2/Client.html))
            #    *  (`GRPC::Core::Channel`) a gRPC channel with included credentials
            #    *  (`GRPC::Core::ChannelCredentials`) a gRPC credentails object
            #    *  (`nil`) indicating no credentials
            #   @return [::Object]
            # @!attribute [rw] scope
            #   The OAuth scopes
            #   @return [::Array<::String>]
            # @!attribute [rw] lib_name
            #   The library name as recorded in instrumentation and logging
            #   @return [::String]
            # @!attribute [rw] lib_version
            #   The library version as recorded in instrumentation and logging
            #   @return [::String]
            # @!attribute [rw] channel_args
            #   Extra parameters passed to the gRPC channel. Note: this is ignored if a
            #   `GRPC::Core::Channel` object is provided as the credential.
            #   @return [::Hash]
            # @!attribute [rw] interceptors
            #   An array of interceptors that are run before calls are executed.
            #   @return [::Array<::GRPC::ClientInterceptor>]
            # @!attribute [rw] timeout
            #   The call timeout in seconds.
            #   @return [::Numeric]
            # @!attribute [rw] metadata
            #   Additional gRPC headers to be sent with the call.
            #   @return [::Hash{::Symbol=>::String}]
            # @!attribute [rw] retry_policy
            #   The retry policy. The value is a hash with the following keys:
            #    *  `:initial_delay` (*type:* `Numeric`) - The initial delay in seconds.
            #    *  `:max_delay` (*type:* `Numeric`) - The max delay in seconds.
            #    *  `:multiplier` (*type:* `Numeric`) - The incremental backoff multiplier.
            #    *  `:retry_codes` (*type:* `Array<String>`) - The error codes that should
            #       trigger a retry.
            #   @return [::Hash]
            # @!attribute [rw] quota_project
            #   A separate project against which to charge quota.
            #   @return [::String]
            #
            class Configuration
              extend ::Gapic::Config

              config_attr :endpoint,      "orgpolicy.googleapis.com", ::String
              config_attr :credentials,   nil do |value|
                allowed = [::String, ::Hash, ::Proc, ::Symbol, ::Google::Auth::Credentials, ::Signet::OAuth2::Client, nil]
                allowed += [::GRPC::Core::Channel, ::GRPC::Core::ChannelCredentials] if defined? ::GRPC
                allowed.any? { |klass| klass === value }
              end
              config_attr :scope,         nil, ::String, ::Array, nil
              config_attr :lib_name,      nil, ::String, nil
              config_attr :lib_version,   nil, ::String, nil
              config_attr(:channel_args,  { "grpc.service_config_disable_resolution" => 1 }, ::Hash, nil)
              config_attr :interceptors,  nil, ::Array, nil
              config_attr :timeout,       nil, ::Numeric, nil
              config_attr :metadata,      nil, ::Hash, nil
              config_attr :retry_policy,  nil, ::Hash, ::Proc, nil
              config_attr :quota_project, nil, ::String, nil

              # @private
              def initialize parent_config = nil
                @parent_config = parent_config unless parent_config.nil?

                yield self if block_given?
              end

              ##
              # Configurations for individual RPCs
              # @return [Rpcs]
              #
              def rpcs
                @rpcs ||= begin
                  parent_rpcs = nil
                  parent_rpcs = @parent_config.rpcs if defined?(@parent_config) && @parent_config.respond_to?(:rpcs)
                  Rpcs.new parent_rpcs
                end
              end

              ##
              # Configuration RPC class for the OrgPolicy API.
              #
              # Includes fields providing the configuration for each RPC in this service.
              # Each configuration object is of type `Gapic::Config::Method` and includes
              # the following configuration fields:
              #
              #  *  `timeout` (*type:* `Numeric`) - The call timeout in seconds
              #  *  `metadata` (*type:* `Hash{Symbol=>String}`) - Additional gRPC headers
              #  *  `retry_policy (*type:* `Hash`) - The retry policy. The policy fields
              #     include the following keys:
              #      *  `:initial_delay` (*type:* `Numeric`) - The initial delay in seconds.
              #      *  `:max_delay` (*type:* `Numeric`) - The max delay in seconds.
              #      *  `:multiplier` (*type:* `Numeric`) - The incremental backoff multiplier.
              #      *  `:retry_codes` (*type:* `Array<String>`) - The error codes that should
              #         trigger a retry.
              #
              class Rpcs
                ##
                # RPC-specific configuration for `list_constraints`
                # @return [::Gapic::Config::Method]
                #
                attr_reader :list_constraints
                ##
                # RPC-specific configuration for `list_policies`
                # @return [::Gapic::Config::Method]
                #
                attr_reader :list_policies
                ##
                # RPC-specific configuration for `get_policy`
                # @return [::Gapic::Config::Method]
                #
                attr_reader :get_policy
                ##
                # RPC-specific configuration for `get_effective_policy`
                # @return [::Gapic::Config::Method]
                #
                attr_reader :get_effective_policy
                ##
                # RPC-specific configuration for `create_policy`
                # @return [::Gapic::Config::Method]
                #
                attr_reader :create_policy
                ##
                # RPC-specific configuration for `update_policy`
                # @return [::Gapic::Config::Method]
                #
                attr_reader :update_policy
                ##
                # RPC-specific configuration for `delete_policy`
                # @return [::Gapic::Config::Method]
                #
                attr_reader :delete_policy

                # @private
                def initialize parent_rpcs = nil
                  list_constraints_config = parent_rpcs.list_constraints if parent_rpcs.respond_to? :list_constraints
                  @list_constraints = ::Gapic::Config::Method.new list_constraints_config
                  list_policies_config = parent_rpcs.list_policies if parent_rpcs.respond_to? :list_policies
                  @list_policies = ::Gapic::Config::Method.new list_policies_config
                  get_policy_config = parent_rpcs.get_policy if parent_rpcs.respond_to? :get_policy
                  @get_policy = ::Gapic::Config::Method.new get_policy_config
                  get_effective_policy_config = parent_rpcs.get_effective_policy if parent_rpcs.respond_to? :get_effective_policy
                  @get_effective_policy = ::Gapic::Config::Method.new get_effective_policy_config
                  create_policy_config = parent_rpcs.create_policy if parent_rpcs.respond_to? :create_policy
                  @create_policy = ::Gapic::Config::Method.new create_policy_config
                  update_policy_config = parent_rpcs.update_policy if parent_rpcs.respond_to? :update_policy
                  @update_policy = ::Gapic::Config::Method.new update_policy_config
                  delete_policy_config = parent_rpcs.delete_policy if parent_rpcs.respond_to? :delete_policy
                  @delete_policy = ::Gapic::Config::Method.new delete_policy_config

                  yield self if block_given?
                end
              end
            end
          end
        end
      end
    end
  end
end
